<?php

/**
 * @file
 * Definition of \Drupal\ckeditor\Tests\CKEditorAdminTest.
 */

namespace Drupal\ckeditor\Tests;

use Drupal\editor\Plugin\Core\Entity\Editor;
use Drupal\simpletest\WebTestBase;

/**
 * Tests administration of CKEditor.
 */
class CKEditorAdminTest extends WebTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = array('filter', 'editor', 'ckeditor');

  public static function getInfo() {
    return array(
      'name' => 'CKEditor administration',
      'description' => 'Tests administration of CKEditor.',
      'group' => 'CKEditor',
    );
  }

  function setUp() {
    parent::setUp();

    // Create text format.
    $filtered_html_format = entity_create('filter_format', array(
      'format' => 'filtered_html',
      'name' => 'Filtered HTML',
      'weight' => 0,
      'filters' => array(),
    ));
    $filtered_html_format->save();

    // Create admin user.
    $this->admin_user = $this->drupalCreateUser(array('administer filters'));
  }

  function testAdmin() {
    $manager = drupal_container()->get('plugin.manager.editor');
    $ckeditor = $manager->createInstance('ckeditor');

    $this->drupalLogin($this->admin_user);
    $this->drupalGet('admin/config/content/formats/filtered_html');

    // Ensure no Editor config entity exists yet.
    $editor = entity_load('editor', 'filtered_html');
    $this->assertFalse($editor, 'No Editor config entity exists yet.');

    // Verify the "Text Editor" <select> when a text editor is available.
    $select = $this->xpath('//select[@name="editor[editor]"]');
    $select_is_disabled = $this->xpath('//select[@name="editor[editor]" and @disabled="disabled"]');
    $options = $this->xpath('//select[@name="editor[editor]"]/option');
    $this->assertTrue(count($select) === 1, 'The Text Editor select exists.');
    $this->assertTrue(count($select_is_disabled) === 0, 'The Text Editor select is not disabled.');
    $this->assertTrue(count($options) === 2, 'The Text Editor select has two options.');
    $this->assertTrue(((string) $options[0]) === 'None', 'Option 1 in the Text Editor select is "None".');
    $this->assertTrue(((string) $options[1]) === 'CKEditor', 'Option 2 in the Text Editor select is "CKEditor".');
    $this->assertTrue(((string) $options[0]['selected']) === 'selected', 'Option 1 ("None") is selected.');

    // Select the "CKEditor" editor and click the "Configure" button.
    $edit = array(
      'editor[editor]' => 'ckeditor',
    );
    // $this->drupalPost(NULL, $edit, t('Configure'));
    $this->drupalPostAjax(NULL, $edit, 'editor_configure');
    $editor = entity_load('editor', 'filtered_html');
    $this->assertFalse($editor, 'No Editor config entity exists yet.');

    // Default configuration.
    $default_buttons_value = '[["Source","|","Bold","Italic","|","NumberedList","BulletedList","Blockquote","|","JustifyLeft","JustifyCenter","JustifyRight","|","Link","Unlink","|","Image","Maximize"]]';
    $this->assertFieldByName('editor[settings][toolbar][buttons]', $default_buttons_value, 'Buttons configuration');
    $styles_textarea = $this->xpath('//textarea[@name="editor[settings][plugins][stylescombo][styles]"]');
    $this->assertFieldByXPath('//textarea[@name="editor[settings][plugins][stylescombo][styles]"]', '', 'The styles textarea exists and is empty.');
    $this->assertTrue(count($styles_textarea) === 1, 'The "styles" textarea exists.');
    $this->drupalPost(NULL, $edit, t('Save configuration'));

    // Ensure an Editor object exists now, with the proper settings.
    $expected_settings = $ckeditor->getDefaultSettings();
    $expected_settings['plugins']['stylescombo']['styles'] = '';
    $editor = entity_load('editor', 'filtered_html');
    $this->assertTrue($editor instanceof Editor, 'An Editor config entity exists now.');
    $this->assertIdentical($expected_settings, $editor->settings, 'The Editor config entity has the correct settings.');

    // Configure a plugin.
    $edit = array(
      'editor[settings][plugins][stylescombo][styles]' => "Title|h1.title\nCallout|p.callout\n\n",
    );
    $this->drupalPost(NULL, $edit, t('Save configuration'));
    $expected_settings['plugins']['stylescombo']['styles'] = "Title|h1.title\nCallout|p.callout\n\n";
    $editor = entity_load('editor', 'filtered_html');
    $this->assertTrue($editor instanceof Editor, 'An Editor config entity exists.');
    $this->assertIdentical($expected_settings, $editor->settings, 'The Editor config entity has the correct settings.');

    // "Drag and drop" buttons: remove one button, add a separator, add a button
    // and finally, add a button to a new row.
    $edit = array(
      'editor[settings][toolbar][buttons]' => '[["Source","|","Bold","Italic","|","NumberedList","BulletedList","Blockquote","|","JustifyLeft","JustifyCenter","JustifyRight","|","Link","Unlink","|","Image","|","Undo"],["Redo"]]'
    );
    $this->drupalPost(NULL, $edit, t('Save configuration'));
    $expected_settings['toolbar']['buttons'][0][17] = '|';
    $expected_settings['toolbar']['buttons'][0][18] = 'Undo';
    $expected_settings['toolbar']['buttons'][1][] = 'Redo';
    $editor = entity_load('editor', 'filtered_html');
    $this->assertTrue($editor instanceof Editor, 'An Editor config entity exists.');
    $this->assertIdentical($expected_settings, $editor->settings, 'The Editor config entity has the correct settings.');

    // Now enable the ckeditor_test module, which provides one configurable
    // CKEditor plugin — this should not affect the Editor config entity.
    module_enable(array('ckeditor_test'));
    drupal_container()->get('plugin.manager.ckeditor.plugin')->clearCachedDefinitions();
    $this->drupalGet('admin/config/content/formats/filtered_html');
    $ultra_llama_mode_checkbox = $this->xpath('//input[@type="checkbox" and @name="editor[settings][plugins][llama_contextual_and_button][ultra_llama_mode]" and not(@checked)]');
    $this->assertTrue(count($ultra_llama_mode_checkbox) === 1, 'The "Ultra llama mode" checkbox exists and is not checked.');
    $editor = entity_load('editor', 'filtered_html');
    $this->assertTrue($editor instanceof Editor, 'An Editor config entity exists.');
    $this->assertIdentical($expected_settings, $editor->settings, 'The Editor config entity has the correct settings.');

    // Finally, check the "Ultra llama mode" checkbox.
    $edit = array(
      'editor[settings][plugins][llama_contextual_and_button][ultra_llama_mode]' => '1',
    );
    $this->drupalPost(NULL, $edit, t('Save configuration'));
    $ultra_llama_mode_checkbox = $this->xpath('//input[@type="checkbox" and @name="editor[settings][plugins][llama_contextual_and_button][ultra_llama_mode]" and @checked="checked"]');
    $this->assertTrue(count($ultra_llama_mode_checkbox) === 1, 'The "Ultra llama mode" checkbox exists and is checked.');
    $expected_settings['plugins']['llama_contextual_and_button']['ultra_llama_mode'] = '1';
    $editor = entity_load('editor', 'filtered_html');
    $this->assertTrue($editor instanceof Editor, 'An Editor config entity exists.');
    $this->assertIdentical($expected_settings, $editor->settings, 'The Editor config entity has the correct settings.');
  }

}
