/**
 * @file
 * Text editor-based Create.js widget for processed text content in Drupal.
 *
 * Depends on Editor.module. Works with any (WYSIWYG) editor that implements the
 * attachInlineEditor(), detach() and onChange() methods.
 */
(function (jQuery, Drupal, drupalSettings) {

"use strict";

  jQuery.widget('Drupal.drupalWysiwygWidget', jQuery.Create.editWidget, {

    textFormat: null,
    textFormatHasTransformations: null,
    textEditor: null,

    /**
     * Implements getEditUISettings() method.
     */
    getEditUISettings: function() {
      return { padding: true, unifiedToolbar: true, fullWidthToolbar: true };
    },

    /**
     * Implements jQuery UI widget factory's _init() method.
     *
     * @todo: POSTPONED_ON(Create.js, https://github.com/bergie/create/issues/142)
     * Get rid of this once that issue is solved.
     */
    _init: function() {},

    /**
     * Implements Create's _initialize() method.
     */
    _initialize: function() {
      var propertyID = Drupal.edit.util.calcPropertyID(this.options.entity, this.options.property);
      var metadata = Drupal.edit.metadataCache[propertyID].custom;

      this.textFormat = drupalSettings.editor.formats[metadata.format];
      this.textFormatHasTransformations = metadata.formatHasTransformations;
      this.textEditor = Drupal.editors[this.textFormat.editor];

      this._bindEvents();
    },

    /**
     * Binds to events.
     */
    _bindEvents: function() {
      var that = this;

      // Sets the state to 'activated' upon clicking the element.
      this.element.on('click.edit', function(event) {
        event.stopPropagation();
        event.preventDefault();
        that.options.activating();
      });
    },

    /**
     * Makes this PropertyEditor widget react to state changes.
     */
    stateChange: function(from, to) {
      var that = this;
      switch (to) {
        case 'inactive':
          break;
        case 'candidate':
          if (from !== 'inactive') {
            if (from !== 'highlighted') {
              this.element.attr('contentEditable', 'false');
              this.textEditor.detach(this.element.get(0), this.textFormat);
            }

            this._removeValidationErrors();
            this._cleanUp();
            this._bindEvents();
          }
          break;
        case 'highlighted':
          break;
        case 'activating':
          // When transformation filters have been been applied to the processed
          // text of this field, then we'll need to load a re-rendered version of
          // it without the transformation filters.
          if (this.textFormatHasTransformations) {
            Drupal.edit.util.loadRerenderedProcessedText({
              $editorElement: this.element,
              propertyID: Drupal.edit.util.calcPropertyID(this.options.entity, this.options.property),
              callback: function (rerendered) {
                that.element.html(rerendered);
                that.options.activated();
              }
            });
          }
          // When no transformation filters have been applied: start WYSIWYG
          // editing immediately!
          else {
            this.options.activated();
          }
          break;
        case 'active':
          this.element.attr('contentEditable', 'true');
          this.textEditor.attachInlineEditor(
            this.element.get(0),
            this.textFormat,
            this.toolbarView.getMainWysiwygToolgroupId(),
            this.toolbarView.getFloatedWysiwygToolgroupId()
          );

          // Sets the state to 'changed' whenever the content has changed.
          this.textEditor.onChange(this.element.get(0), function (html) {
            that.options.changed(html);
          });
          break;
        case 'changed':
          break;
        case 'saving':
          this._removeValidationErrors();
          break;
        case 'saved':
          break;
        case 'invalid':
          break;
      }
    },

    /**
     * Removes validation errors' markup changes, if any.
     *
     * @todo: this should not be necessary, will be obviated by edit.module.
     */
    _removeValidationErrors: function() {
      this.element
        .removeClass('edit-validation-error')
        .next('.edit-validation-errors').remove();
    },

    /**
     * Cleans up after the widget has been saved.
     *
     * @todo: this should not be necessary, will be obviated by edit.module.
     */
    _cleanUp: function() {
      Drupal.edit.util.form.unajaxifySaving(jQuery('#edit_backstage form .edit-form-submit'));
      jQuery('#edit_backstage form').remove();
    }
  });

})(jQuery, Drupal, drupalSettings);
